﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DatePlusClasses
{
    // расширенный класс дат, поддержка пустых значений, дополнительные функции
    public class DatePlus
    {
        private DateTime? _nullableDateTimeBase = null;

        public DatePlus() { }

        public DatePlus(DateTime startDateTime)
        {

            _nullableDateTimeBase = startDateTime.Date;
        }

        public DatePlus(string startDateTimeStr)
        {
            if (startDateTimeStr != "")
            {
                _nullableDateTimeBase = Convert.ToDateTime(startDateTimeStr).Date;
            }

        }

        // НАДО ПРОВЕРИТЬ!!! создание из ячейки DataTable, прочитанной из базы 
        public DatePlus(object dbDate)
        {
            if (dbDate != DBNull.Value)
            {
                _nullableDateTimeBase = Convert.ToDateTime(dbDate);
            }
        }

        //The pointer for this method was null.
        public bool Empty
        {
            get
            {
                return !_nullableDateTimeBase.HasValue;
            }
        }

        public bool InFuture
        {
            get
            {
                bool inFuture = false;

                // проверка на будущее имеет смысл только для непустых дат
                if (_nullableDateTimeBase == null) { return inFuture;  } 

                DateTime nowDateTime = DateTime.Now;
                int compareCode = _nullableDateTimeBase.Value.CompareTo(nowDateTime);

                //_nullableDateTimeBase наступает позже nowDateTime
                if (compareCode > 0)
                {
                    inFuture = true;
                }
                return inFuture;
            }
        }

        public DateTime DateTime
        {
            get
            {
                if (_nullableDateTimeBase != null)
                {
                    return _nullableDateTimeBase.Value;
                }
                else
                {
                    throw new Exception("Невозможно вернуть DateTime так как дата пустая.");
                }
            }
        }

        // строка даты для ms sql  не зависящая от региональных настроек
        public string SqlDateString
        {
            get
            {
                string universalSqlString = "";

                if (!_nullableDateTimeBase.HasValue) 
                {
                    throw new Exception("Невозможно создать универсальную строку даты для MS Sql из пустой даты!"); 
                }

                universalSqlString = _nullableDateTimeBase.Value.ToString("yyyyMMdd");
                return universalSqlString;
            }
        }

        // DateTime или DBNull.Value
        public object DBDateObject
        {
            get
            {
                object dbDateObj = null;
                if (_nullableDateTimeBase.HasValue)
                {
                    dbDateObj = _nullableDateTimeBase.Value;
                }
                else
                {
                    dbDateObj = DBNull.Value;
                }
                return dbDateObj;
            }
        }

        public string ShortString
        {
            get
            {
                string shortString = "";
                if (_nullableDateTimeBase != null)
                {
                    shortString = _nullableDateTimeBase.Value.ToShortDateString();
                }
                return shortString;
            }
        }

        public string LongString
        {
            get
            {
                string longString = "";
                if (_nullableDateTimeBase != null)
                {
                    longString = _nullableDateTimeBase.Value.ToLongDateString();
                }
                return longString;
            }
        }

        public bool After(DatePlus dateToCompare)
        {
            bool after = false;

            if (this.Empty || dateToCompare.Empty) { return after; }

            int compareCode = _nullableDateTimeBase.Value.CompareTo(dateToCompare.DateTime);

            //_nullableDateTimeBase наступает позже dateToCompare
            if (compareCode > 0)
            {
                after = true;
            }

            return after;
        }

        public bool Before(DatePlus dateToCompare)
        {
            bool before = false;

            if (this.Empty || dateToCompare.Empty) { return before; }

            int compareCode = _nullableDateTimeBase.Value.CompareTo(dateToCompare.DateTime);

            //_nullableDateTimeBase наступает раньше 
            if (compareCode < 0)
            {
                before = true;
            }

            return before;
        }

        public bool Equal(DatePlus dateToCompare)
        {
            bool equal = false;

            if (this.Empty || dateToCompare.Empty) { return equal; }

            int compareCode = _nullableDateTimeBase.Value.CompareTo(dateToCompare.DateTime);

            //_nullableDateTimeBase наступает раньше 
            if (compareCode < 0)
            {
                equal = true;
            }

            return equal;
        }

        public override string ToString()
        {
            return ShortString;
        }

        public static bool DateCorrect(string dateString)
        {
            bool dateCorrect = true;
            if (dateString == "") { return dateCorrect;  }
            try
            {
                Convert.ToDateTime(dateString);
            }
            catch
            {
                dateCorrect = false;
            }
            return dateCorrect;
        }

        // end  class DatePlus
    }
}
