﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DatePlusClasses
{
    public partial class DatePlusPicker : UserControl
    {
        private CalendarForm _calendarForm = null;
        private DatePlus _date = null;
        private MessageForm _wrongDateBox = null;

        private bool _allowFuture = false;

        [Description("AllowFuture"), Category("Behavior")] 
        public bool AllowFuture
        {
            get { return _allowFuture; }
            set { _allowFuture = value; }
        }


        public string EnteredDateString  
        { 
            get  
            {
                string enteredString = "";
                if (_dateMaskedTextBox.Text != "  .  .")
                {
                    enteredString = _dateMaskedTextBox.Text;
                }
                return enteredString;  
            }
        }

        public DatePlus Date
        {
            get 
            {
                if (DateCorrect)
                {
                    _date = new DatePlus(EnteredDateString);
                }
                else
                {
                    _date = new DatePlus();
                }
                return _date; 
            }
            set 
            { 
                _date = value;
                _dateMaskedTextBox.Text = _date.ShortString;
            }
        }

        public bool DateCorrect
        {
            get
            {
                bool dateCorrect = false;

                dateCorrect = DatePlus.DateCorrect(EnteredDateString);

                if (dateCorrect  && !AllowFuture)
                {
                    DatePlus date = new DatePlus(EnteredDateString);
                    if (date.InFuture)
                    {
                        dateCorrect = false;
                    }
                }

                return dateCorrect;
            }
        }

        public bool Empty
        {
            get
            {
                bool empty = false;
                if (EnteredDateString == "")
                {
                    empty = true;
                }
                return empty;
            }
        }
        
        // КОНСТРУКТОР
        public DatePlusPicker()
        {
            InitializeComponent();
            _date = new DatePlus();
            _calendarForm = new CalendarForm();
            _wrongDateBox = new MessageForm();
            
            _wrongDateBox.Message = "Дата введена неправильно! Введите правильную или пустую дату!";

            _calendarForm.Calendar.DateSelected += new DateRangeEventHandler(Calendar_DateSelected);
            _calendarButton.BackgroundImage = _arrowsImageList.Images["arrow-01-down.png"];

            _calendarForm.VisibleChanged += new EventHandler(_calendarForm_VisibleChanged);
        }

        void _calendarForm_VisibleChanged(object sender, EventArgs e)
        {
            _calendarForm.Font = new Font(_calendarForm.Font.Name, this.Font.Size);
            if (_calendarForm.Visible)
            {
                _calendarButton.BackgroundImage = _arrowsImageList.Images["arrow-01-up.png"];
            }
            else
            {
                _calendarButton.BackgroundImage = _arrowsImageList.Images["arrow-01-down.png"];
            }
        }


        // при выборе даты в календаре ставим строку в MaskedTextBox
        void Calendar_DateSelected(object sender, DateRangeEventArgs e)
        {
            this._dateMaskedTextBox.Text = _calendarForm.Calendar.SelectionEnd.ToShortDateString();
        }


        private void _calendarButton_Click(object sender, EventArgs e)
        {
            if (DateCorrect && !Empty && Date.DateTime > _calendarForm.Calendar.MinDate && Date.DateTime < _calendarForm.Calendar.MaxDate)
            {
                _calendarForm.Calendar.SelectionStart = Date.DateTime;
                _calendarForm.Calendar.SelectionEnd = Date.DateTime;
            }

            Point parentScreenPoint = this.Parent.PointToScreen(new Point(this.Location.X, this.Location.Y + this.Height));
            _calendarForm.Location = parentScreenPoint;

            if (_calendarForm.Visible)
            {
                _calendarForm.Hide();
            }
            else
            {
                _calendarForm.Show();
            }
        }

        private void _dateMaskedTextBox_Validated(object sender, EventArgs e)
        {
            showWrongDateMessageBox();
        }

        private void _dateMaskedTextBox_MouseLeave(object sender, EventArgs e)
        {          
            showWrongDateMessageBox();
        }


        private void showWrongDateMessageBox()
        {

            if (!DateCorrect && !_wrongDateBox.Visible)
            {
                _wrongDateBox.ShowDialog();
                _dateMaskedTextBox.Focus();
                _dateMaskedTextBox.SelectAll();
            }
            
        }


        // при изменении текста обновляем введенную строку и признак правильности даты
        private void _dateMaskedTextBox_TextChanged(object sender, EventArgs e)
        {
            
        }

        // end public partial class DatePlusPicker : UserControl
    }

}
